#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define F_CPU 16000000
#define SPEED 10
#define SENSORS 8
#define NOT_YET -1

int main() {
	int left_limit, right_limit, sensor;
	unsigned char sensor_input;
    /*
    Init port for sensonrs
	*/
	PORTA = 0xff;
	DDRA = 0x00;
	/*
	Init port for L298N and H-bridge controls
	*/
	PORTD = 0xff;
	DDRD = 0xff;
	
	int speed_mult_factor;
	int streer amplitude;
	
	while (1) {
		/* 
		8 sensors
		Input is PINA
		*/
		sensor_input = PINA;
		
		/* 
		Init left_deviation & right_deviation 
		*/
		left_limit 	= NOT_YET;
		right_limit = NOT_YET;
		
		/* 
		Cycle through the values of the 8 sensors and
		find the left limit of the line
		*/
		for (sensor = 0; sensor < SENSORS; sensor++){
			if (bit_is_set(sensor_input, sensor)){
				/*
				Set the left limit of the black line
				*/
				left_limit = sensor;
			}
		}
		
		/* 
		Cycle through the values of the 8 sensors and
		find the right limit of the line
		*/
		for (sensor = 0; sensor < SENSORS; sensor++){
			if (bit_is_set(sensor_input, sensor)){
				/*
				Set the right limit of the black line
				*/
				right_limit = sensor;
			}
		}
		
		/*
		Using the limits of the lines found out from the 
		sensors the bot decides it's next move
		
		We've considered 3 cases

		Case 0: If there is no line, don't move
		*/
		if (left_limit == NOT_YET && right_limit == NOT_YET){
			/*
			Send the stop commands to the H bridge
			*/
			PORTD = 0x00;
			OC1A = 0x00;
			continue;
		}		
		
		/*
		Case 1: The line centered
		*/
		if (right_limit + right_limit / 2 == 4) {
			/*
			Full speed ahead
			*/
			PORTD = 0x55;
			OC1A = SPEED * 10;
			continue;
		}
		
		/*
		Case 2: The line is to the right
		*/
		if (right_limit <= 3) {
			/*
			Steer to the left
			*/
			PORTD = 0x66;
			/*
			The more it has to steer the more it decelerates
			*/
			OC1A = SPEED * right_limit;
			continue;
		}
		
		/*
		Case 3: The line is to the left
		*/
		if (right_limit >= 5) {
			/*
			Steer to the right
			*/
			PORTD = 0x99;
			/*
			The more it has to steer the more it decelerates
			*/
			OC1A = SPEED * (7 - left_limit);
			continue;
		}
		/*
		Wait for the robot to move according to the
		instructions.
		It makes sure it does not get stuck in a circle
		*/
		_delay_ms(500);
	}
	return 0;
}
